@@ -1,7 +1,7 @@
 /*
 htop - DateTimeMeter.c
 (C) 2004-2020 Hisham H. Muhammad, Michael Schönitzer
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
@@ -13,8 +13,8 @@ in the source distribution for its full text.
 #include <sys/time.h>
 
 #include "CRT.h"
+#include "Machine.h"
 #include "Object.h"
-#include "ProcessList.h"
 
 
 static const int DateTimeMeter_attributes[] = {
@@ -22,10 +22,10 @@ static const int DateTimeMeter_attributes[] = {
 };
 
 static void DateTimeMeter_updateValues(Meter* this) {
-   const ProcessList* pl = this->pl;
+   const Machine* host = this->host;
 
    struct tm result;
-   const struct tm* lt = localtime_r(&pl->realtime.tv_sec, &result);
+   const struct tm* lt = localtime_r(&host->realtime.tv_sec, &result);
    int year = lt->tm_year + 1900;
    if (((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0)) {
       this->total = 366;
