@@ -3,7 +3,7 @@
 /*
 htop - ColorsPanel.h
 (C) 2021 htop dev team
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
