@@ -1,27 +1,28 @@
 /*
 htop - HostnameMeter.c
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPL, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include "config.h" // IWYU pragma: keep
+
 #include "HostnameMeter.h"
 
 #include "CRT.h"
-
-#include <unistd.h>
+#include "Object.h"
+#include "Platform.h"
 
 
-int HostnameMeter_attributes[] = {
+static const int HostnameMeter_attributes[] = {
    HOSTNAME
 };
 
-static void HostnameMeter_updateValues(Meter* this, char* buffer, int size) {
-   (void) this;
-   gethostname(buffer, size-1);
+static void HostnameMeter_updateValues(Meter* this) {
+   Platform_getHostname(this->txtBuffer, sizeof(this->txtBuffer));
 }
 
-MeterClass HostnameMeter_class = {
+const MeterClass HostnameMeter_class = {
    .super = {
       .extends = Class(Meter),
       .delete = Meter_delete
