@@ -1,3 +1,11 @@
+/*
+htop - InfoScreen.c
+(C) 2016 Hisham H. Muhammad
+(C) 2020,2022 htop dev team
+Released under the GNU GPLv2+, see the COPYING file
+in the source distribution for its full text.
+*/
+
 #include "config.h" // IWYU pragma: keep
 
 #include "InfoScreen.h"
@@ -57,13 +65,13 @@ void InfoScreen_drawTitled(InfoScreen* this, const char* fmt, ...) {
    attrset(CRT_colors[DEFAULT_COLOR]);
    Panel_draw(this->display, true, true, true, false);
 
-   IncSet_drawBar(this->inc);
+   IncSet_drawBar(this->inc, CRT_colors[FUNCTION_BAR]);
 }
 
 void InfoScreen_addLine(InfoScreen* this, const char* line) {
    Vector_add(this->lines, (Object*) ListItem_new(line, 0));
    const char* incFilter = IncSet_filter(this->inc);
-   if (!incFilter || String_contains_i(line, incFilter)) {
+   if (!incFilter || String_contains_i(line, incFilter, true)) {
       Panel_add(this->display, Vector_get(this->lines, Vector_size(this->lines) - 1));
    }
 }
@@ -72,7 +80,7 @@ void InfoScreen_appendLine(InfoScreen* this, const char* line) {
    ListItem* last = (ListItem*)Vector_get(this->lines, Vector_size(this->lines) - 1);
    ListItem_append(last, line);
    const char* incFilter = IncSet_filter(this->inc);
-   if (incFilter && Panel_get(this->display, Panel_size(this->display) - 1) != (Object*)last && String_contains_i(line, incFilter)) {
+   if (incFilter && Panel_get(this->display, Panel_size(this->display) - 1) != (Object*)last && String_contains_i(line, incFilter, true)) {
       Panel_add(this->display, (Object*)last);
    }
 }
@@ -89,15 +97,9 @@ void InfoScreen_run(InfoScreen* this) {
    while (looping) {
 
       Panel_draw(panel, false, true, true, false);
-      IncSet_drawBar(this->inc);
+      IncSet_drawBar(this->inc, CRT_colors[FUNCTION_BAR]);
 
-      if (this->inc->active) {
-         (void) move(LINES - 1, CRT_cursorX);
-      }
-#ifdef HAVE_SET_ESCDELAY
-      set_escdelay(25);
-#endif
-      int ch = getch();
+      int ch = Panel_getCh(panel);
 
       if (ch == ERR) {
          if (As_InfoScreen(this)->onErr) {
@@ -136,48 +138,48 @@ void InfoScreen_run(InfoScreen* this) {
       }
 
       switch (ch) {
-      case ERR:
-         continue;
-      case KEY_F(3):
-      case '/':
-         IncSet_activate(this->inc, INC_SEARCH, panel);
-         break;
-      case KEY_F(4):
-      case '\\':
-         IncSet_activate(this->inc, INC_FILTER, panel);
-         break;
-      case KEY_F(5):
-         clear();
-         if (As_InfoScreen(this)->scan) {
-            Vector_prune(this->lines);
-            InfoScreen_scan(this);
-         }
-
-         InfoScreen_draw(this);
-         break;
-      case '\014': // Ctrl+L
-         clear();
-         InfoScreen_draw(this);
-         break;
-      case 27:
-      case 'q':
-      case KEY_F(10):
-         looping = false;
-         break;
-      case KEY_RESIZE:
-         Panel_resize(panel, COLS, LINES - 2);
-         if (As_InfoScreen(this)->scan) {
-            Vector_prune(this->lines);
-            InfoScreen_scan(this);
-         }
-
-         InfoScreen_draw(this);
-         break;
-      default:
-         if (As_InfoScreen(this)->onKey && InfoScreen_onKey(this, ch)) {
+         case ERR:
             continue;
-         }
-         Panel_onKey(panel, ch);
+         case KEY_F(3):
+         case '/':
+            IncSet_activate(this->inc, INC_SEARCH, panel);
+            break;
+         case KEY_F(4):
+         case '\\':
+            IncSet_activate(this->inc, INC_FILTER, panel);
+            break;
+         case KEY_F(5):
+            clear();
+            if (As_InfoScreen(this)->scan) {
+               Vector_prune(this->lines);
+               InfoScreen_scan(this);
+            }
+
+            InfoScreen_draw(this);
+            break;
+         case '\014': // Ctrl+L
+            clear();
+            InfoScreen_draw(this);
+            break;
+         case 27:
+         case 'q':
+         case KEY_F(10):
+            looping = false;
+            break;
+         case KEY_RESIZE:
+            Panel_resize(panel, COLS, LINES - 2);
+            if (As_InfoScreen(this)->scan) {
+               Vector_prune(this->lines);
+               InfoScreen_scan(this);
+            }
+
+            InfoScreen_draw(this);
+            break;
+         default:
+            if (As_InfoScreen(this)->onKey && InfoScreen_onKey(this, ch)) {
+               continue;
+            }
+            Panel_onKey(panel, ch);
       }
    }
 }
