@@ -3,14 +3,17 @@
 /*
 htop - DarwinProcess.h
 (C) 2015 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
 #include <sys/sysctl.h>
 
-#include "Settings.h"
-#include "darwin/DarwinProcessList.h"
+#include "Machine.h"
+#include "darwin/DarwinProcessTable.h"
+
+
+#define PROCESS_FLAG_TTY 0x00000100
 
 typedef struct DarwinProcess_ {
    Process super;
@@ -25,13 +28,13 @@ extern const ProcessClass DarwinProcess_class;
 
 extern const ProcessFieldData Process_fields[LAST_PROCESSFIELD];
 
-Process* DarwinProcess_new(const Settings* settings);
+Process* DarwinProcess_new(const Machine* settings);
 
 void Process_delete(Object* cast);
 
 void DarwinProcess_setFromKInfoProc(Process* proc, const struct kinfo_proc* ps, bool exists);
 
-void DarwinProcess_setFromLibprocPidinfo(DarwinProcess* proc, DarwinProcessList* dpl, double timeIntervalNS);
+void DarwinProcess_setFromLibprocPidinfo(DarwinProcess* proc, DarwinProcessTable* dpt, double timeIntervalNS);
 
 /*
  * Scan threads for process state information.
