@@ -4,42 +4,44 @@
 htop - SolarisProcess.h
 (C) 2015 Hisham H. Muhammad
 (C) 2017,2018 Guy M. Broome
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
-#include "Settings.h"
 #include <zone.h>
 #include <sys/proc.h>
+
+/* On OmniOS /usr/include/sys/regset.h redefines ERR to 13 - \r, breaking the Enter key.
+ * Since ncruses macros use the ERR macro, we can not use another name.
+ */
+#undef ERR
 #include <libproc.h>
+#undef ERR
+#define ERR (-1)
+
+#include "Machine.h"
+
 
 typedef struct SolarisProcess_ {
    Process    super;
-   int        kernel;
    zoneid_t   zoneid;
    char*      zname;
    taskid_t   taskid;
    projid_t   projid;
    poolid_t   poolid;
    ctid_t     contid;
-   bool       is_lwp;
    pid_t      realpid;
    pid_t      realppid;
+   pid_t      realtgid;
    pid_t      lwpid;
 } SolarisProcess;
 
-#define Process_isKernelThread(_process) (_process->kernel == 1)
-
-#define Process_isUserlandThread(_process) (_process->pid != _process->tgid)
-
 extern const ProcessClass SolarisProcess_class;
 
 extern const ProcessFieldData Process_fields[LAST_PROCESSFIELD];
 
-Process* SolarisProcess_new(const Settings* settings);
+Process* SolarisProcess_new(const Machine* host);
 
 void Process_delete(Object* cast);
 
-bool Process_isThread(const Process* this);
-
 #endif
